/** 
 *   Test: vac configuration
 *
 *   (c) Copyright 2006 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   29/07/2006: Created.
 **/

#include "vac.h"

#define PRINT_REG(time_secs, press, instr, common_output) {\
        int __i;\
        fprintf(common_output, "%10.2f", time_secs);\
        for(__i=0;__i<instr;__i++) fprintf(common_output," %5.1f", press[__i]);\
        fprintf(common_output, "\n");\
        fflush(common_output);\
        }

#define DIFTV(t1, t0) (\
(double) (t1.tv_sec - t0.tv_sec) + \
(double) (t1.tv_usec - t0.tv_usec) / 1000000.0)


const char *help_text[] ={
#       ifdef VERBOSE_MODE
  "  -v\t\t - Verbose mode (not implemented yet).",
#       endif
  "  -p\t\t - Print parsed configuration file to stdout.",
  "  -w time\t - Wait until saturation time is apply (sec)",
  "  conf_file\t - Config file. If it is omited, read from stdin.",
  "  -d\t\t - Log readings to a file.",
  "  -h\t\t - This help !!!.",
  ""
};


int if_dif_temp(double *press0, double *press1, int instr)
{
  int i;
  int ret = 0;
  
  for(i = 0; i < instr; i++) {
    if (press0[i] != press1[i]) {
      ret = 1;
      break;
    }
  }
  
  return ret;
}

int make_log_header (struct vac *vac_conf, int instr, FILE *common_output)
{
  struct vac *v;
  char units[6];
  int i,j,k;
  double valor;
  time_t now;
  struct tm date;

  // Allocate vac structure
  v = vac_conf;
  
  // Timestamp
  time(&now);                     // Takes timestamp in seconds 
  localtime_r(&now, &date);       // Converts seconds into a more-readable date
  date.tm_mon = date.tm_mon++;
  date.tm_year = date.tm_year + 1900;

  fprintf(common_output, "# VACIO LOG HEADER\n#\n");
  fprintf(common_output, "# DATE: %04d%02d%02d\n", date.tm_year, date.tm_mon,
          date.tm_mday);
  fprintf(common_output, "# TIME: %02d%02d%02d\n#\n", date.tm_hour,
          date.tm_min, date.tm_sec);
  fprintf(common_output, "# INSTRUMENTS: %d\n#\n", instr);
  i=0;
  while (v->next != NULL){
    fprintf(common_output, "# instrument: %d, name: %-9s, dev: %s, ", 
	    i, v->name, v->device);
    fprintf(common_output, "sensors: %d ->", v->sensor_n);
    for (j = 0; j < v->sensor_n; j++){
      fprintf(common_output, " {%s}", v->sensor[j]);
      fprintf(common_output, "\n");
      sensor_get (v->fd, v->sensor[j], &valor, units);
    }
    fprintf(common_output, "#\t      UNITS: ");
    for (k = 0; k<6; k++)
      fprintf(common_output, "%c", units[k]);
    fprintf(common_output, "\n#\n#\n");
    v = v->next;
    i++;
  }
  return 0;
}

int help(void)
{
  int i=0;
  
  while((char) *help_text[i] != 0) fprintf(stderr,"%s\n",help_text[i++]);
  fprintf(stderr,"\n");
  
  return 0;
}


int main(int argc, char * const argv[])
{
  int op;
  int print_flag = 0;
  int daemon_flag = 0;
  char *filename = NULL;
  char *std_in = "/dev/stdin";
  struct configuration *conf = NULL;
  //struct configuration *tag = NULL;
  //char *tagname = NULL;
  struct vac *vac_conf = NULL;
  //  struct vac *v;
  int instr;
  int total_sensors;
  unsigned long interval;
  //int i /*,j*/;
  double *press0, *press1;
  
  time_t timestamp;
  struct tm date;
  FILE *fp = NULL;
  char logfile[18];
  FILE *my_stdout = stdout;
  
  double now_t0;
  int done = 0;
  struct timeval t0, last, now, prev, int_sleep;
  int skip = 0;
  // int tl0, tl1;
  int count;
  int print;
  int stop_flag = 0;
  time_t wait_until_stop = 0;
  time_t saturation_time = 0;
  
  SET_VERBOSE_FLAG(PERR);
  while  ((op = getopt(argc, argv, "vdt:pP:e:s:w:")) != EOF)
    switch( op ){
    case 'v':
      INCREMENT_VERBOSE_FLAG();
      break;
    case 'p':
      print_flag = 1;
      break;
    case 'd':
      daemon_flag = 1;
      break;
    case 's':
      stop_flag = 1;
      saturation_time = (time_t) atol(optarg);
      break;
    case 'w':
      stop_flag = 1;
      wait_until_stop = (time_t) atol(optarg);
      break;
    case 'h':
    default:
      printf("Use %s [-v (incr.)][-p] [conf_file]\n",
	     argv[0]);
      help();
      exit(-1);
    }
  filename = argv[optind];
  if (!filename) filename = std_in;
  if (filename) {
    read_conf_file(&conf, filename);
    interval = get_interval (conf);
    if( expand_conf_braket_array(conf) != 0 ) {
      PVERB(PERR,"%s: bracket not match.\n", filename);
    }
    if (print_flag)
      write_conf_file_flag(conf, "/dev/stdout", 0);
    
    /* vac_conf if NULL hear */
    instr = make_vac_from_conf (&vac_conf, conf);
    
    /* count how many sensors do we have */
    total_sensors = count_sensors(vac_conf);

    free_conf(conf); /* or at end :-) */
    
    if (vac_init(&vac_conf) < 0){
      printf("ERROR opening devices\n");
      exit (-1);
    }
    if (daemon_flag == 1){
      time(&timestamp);
      localtime_r(&timestamp, &date);
      date.tm_mon = date.tm_mon++;
      date.tm_year = date.tm_year + 1900;
      
      sprintf(logfile, "vacio.%4d%02d%02d%02d%02d", date.tm_year, date.tm_mon,
	      date.tm_mday, date.tm_hour, date.tm_min);

      fp = fopen(logfile, "w");

      my_stdout = fp;
      
    }

    make_log_header (vac_conf, instr, my_stdout);
    
    /* Allocate memory for store array values type double, a total of
       total_sensors elements per array */

    press0 = (double *) malloc(sizeof(double) * total_sensors);
    press1 = (double *) malloc(sizeof(double) * total_sensors);

    get_readings (vac_conf, press0);
    
    gettimeofday(&t0, NULL);
    
    memcpy (&last, &t0, sizeof(struct timeval));

    PRINT_REG(0.0, press0, total_sensors, my_stdout);
    gettimeofday (&now, NULL);
    count = 0;
      
    while(!done) {
      if(interval >= 100000) {
	unsigned long time;
	gettimeofday(&int_sleep, NULL);
	time = interval - 10000 - (unsigned long)
	  (DIFTV(int_sleep, t0) * 1000000.0 -
	   (double) count * (double)interval);
	//printf("sleeping: %ld usec\n", time);
	if(time < 10000000)
	  usleep(time);
	else
	  usleep(interval - DIFTV(int_sleep, now) * 1000000.0 
	   - 10000);
      }
      gettimeofday(&now, NULL);
      get_readings (vac_conf,  press1);

      now_t0 = DIFTV(now, t0);
      print = 0;
      if (if_dif_temp(press1, press0, total_sensors)){
	if (skip > 0) {
	  PRINT_REG(DIFTV(prev, t0), press0, total_sensors, my_stdout);
	}
	//printf("skip: %d ", skip);
	memcpy(&last, &now, sizeof(struct timeval));
	//memcpy(press0, press1, sizeof(double) * instr);
	memcpy(press0, press1, sizeof(double) * total_sensors);
	//PRINT_REG(now_t0, press1, instr, my_stdout);
	PRINT_REG(now_t0, press1, total_sensors, my_stdout);
	print = 1;
	skip = 0;
      }
      else {
	memcpy(&prev, &now, sizeof(struct timeval));
	skip++;
      }
      
      if (stop_flag &&
	  (time_t) now_t0 >= wait_until_stop &&
	  (time_t) DIFTV(now, last) >= saturation_time   ) {
	done = 1;
      }
      count++;
    }
    
    free(press0);
    free(press1);
    vac_close(vac_conf);
    free_vac(vac_conf);
    fclose(my_stdout);
  }
  return 0;
}
